// config.php
<?php
$excludeDirs = array('.', '..', 'dashboard', 'img', 'webalizer', 'security');
$htdocsPath = __DIR__;

// Funktion zum Prüfen ob ein Verzeichnis ein Projekt ist
function isProjectDir($dir) {
    global $excludeDirs, $htdocsPath;
    return is_dir($htdocsPath . '/' . $dir) 
           && !in_array($dir, $excludeDirs) 
           && !str_starts_with($dir, '.');
}

// Funktion zum Lesen der Projektmetadaten
function getProjectMetadata($projectPath) {
    $metadataFile = $projectPath . '/project.json';
    if (file_exists($metadataFile)) {
        $metadata = json_decode(file_get_contents($metadataFile), true);
        return [
            'title' => $metadata['title'] ?? basename($projectPath),
            'description' => $metadata['description'] ?? 'Keine Beschreibung verfügbar'
        ];
    }
    return [
        'title' => basename($projectPath),
        'description' => 'Keine Beschreibung verfügbar'
    ];
}

// Hole alle Projekte mit Metadaten
$projects = array_map(function($dir) use ($htdocsPath) {
    $projectPath = $htdocsPath . '/' . $dir;
    $metadata = getProjectMetadata($projectPath);
    return [
        'dir' => $dir,
        'path' => $projectPath,
        'title' => $metadata['title'],
        'description' => $metadata['description'],
        'lastModified' => date("d.m.Y H:i", filemtime($projectPath)),
        'hasIndex' => file_exists($projectPath . '/index.php') || 
                     file_exists($projectPath . '/index.html')
    ];
}, array_filter(scandir($htdocsPath), 'isProjectDir'));
?>