@echo off
chcp 65001 > nul
setlocal EnableDelayedExpansion

:: ==============================
:: XAMPP Projekt Manager
:: ==============================

:: Pfade definieren
set apacheBin=C:\xampp\apache\bin
set apacheConf=C:\xampp\apache\conf
set htdocs=C:\xampp\htdocs
set hostsFile=C:\Windows\System32\drivers\etc\hosts
set vhostsFile=%apacheConf%\extra\httpd-vhosts.conf
set sslConfFile=%apacheConf%\extra\httpd-ssl.conf
set excludeDirs=dashboard img webalizer security

:: Standard Ports
set httpPort=80
set httpsPort=443

:: Menü anzeigen
:menu
cls 
echo ================================
echo XAMPP Projekt Manager
echo ================================
echo.
echo 1) Neues Projekt anlegen
echo 2) Bestehende Projekte scannen
echo 3) Einzelnes Projekt aktualisieren
echo 4) Port-Check durchführen
echo 5) Beenden
echo.
set /p choice="Bitte wählen Sie eine Option (1-5): "

if "%choice%"=="1" goto newProject
if "%choice%"=="2" goto scanProjects
if "%choice%"=="3" goto updateProject
if "%choice%"=="4" goto portCheck
if "%choice%"=="5" exit /b 0
goto menu

:portCheck
cls
echo === Port-Check Diagnose ===
echo.
echo 🔍 Prüfe detailliert Port 80...

:: Genauere Port-Überprüfung
netstat -ano | findstr /R /C:"[^0-9]80[^0-9].*LISTENING" > port_check.txt
if exist port_check.txt (
    set "portInUse="
    for /f "tokens=5" %%a in (port_check.txt) do (
        if not "%%a"=="0" (
            set "portInUse=1"
            echo Gefundene Prozesse auf Port 80:
            echo PID: %%a
            tasklist /fi "pid eq %%a" /fo list | findstr "Bildname:"
            echo.
        )
    )
    if not defined portInUse (
        echo Port 80 ist frei.
    )
    del port_check.txt
) else (
    echo Port 80 ist frei.
)

echo.
echo 🔍 Prüfe bekannte Dienste...
sc query "W3SVC" | findstr "STATE" && echo IIS gefunden
sc query "WMSvc" | findstr "STATE" && echo Web Management Service gefunden
sc query "MSSQLSERVER" | findstr "STATE" && echo SQL Server gefunden
sc query "MySQL" | findstr "STATE" && echo MySQL gefunden

echo.
echo Mögliche Lösungen falls Port belegt:
echo 1) XAMPP komplett beenden und neu starten
echo 2) IIS stoppen: net stop w3svc
echo 3) Alternative Ports verwenden (8080/8443)
echo.
pause
goto menu

:scanProjects
echo.
echo 🔍 Scanne nach Projekten...
echo.

for /d %%D in ("%htdocs%\*") do (
    set "dirname=%%~nxD"
    set "skip="
    
    for %%E in (%excludeDirs%) do (
        if "!dirname!"=="%%E" set "skip=1"
    )
    
    if not defined skip (
        if not exist "%%D\project.json" (
            echo Projekt gefunden: !dirname!
            set /p createJson="Möchten Sie eine project.json erstellen? (j/n): "
            if /i "!createJson!"=="j" (
                set /p title="Titel für !dirname!: "
                set /p desc="Beschreibung: "
                (
                    echo {
                    echo     "title": "!title!",
                    echo     "description": "!desc!"
                    echo }
                ) > "%%D\project.json"
                echo ✅ project.json erstellt für !dirname!
                echo.
            )
        ) else (
            echo ℹ️ !dirname! hat bereits eine project.json
        )
    )
)

echo.
echo ✅ Scan abgeschlossen
pause
goto menu

:updateProject
echo.
echo 📁 Verfügbare Projekte:
echo.
set count=0
for /d %%D in ("%htdocs%\*") do (
    set "dirname=%%~nxD"
    set "skip="
    
    for %%E in (%excludeDirs%) do (
        if "!dirname!"=="%%E" set "skip=1"
    )
    
    if not defined skip (
        set /a count+=1
        echo !count!^) !dirname!
        set "project!count!=%%D"
    )
)

if %count%==0 (
    echo Keine Projekte gefunden.
    pause
    goto menu
)

echo.
set /p projectNum="Wählen Sie ein Projekt (1-%count%): "

set "selectedProject=!project%projectNum%!"
for %%I in ("!selectedProject!") do set "projectName=%%~nxI"

if exist "!selectedProject!" (
    echo.
    set /p title="Neuer Titel: "
    set /p desc="Neue Beschreibung: "
    (
        echo {
        echo     "title": "!title!",
        echo     "description": "!desc!"
        echo }
    ) > "!selectedProject!\project.json"
    echo ✅ project.json aktualisiert
) else (
    echo ❌ Ungültige Auswahl
)

pause
goto menu

:newProject
:: Ports konfigurieren
echo.
echo Standardports sind 80/443
set /p useAltPorts="Alternative Ports verwenden? (j/n): "
if /i "%useAltPorts%"=="j" (
    set httpPort=8080
    set httpsPort=8443
)

:: Firewall-Check
netsh advfirewall show allprofiles state | findstr /C:"Status" > nul
if %errorlevel% neq 0 (
    echo ⚠️ Firewall-Status konnte nicht überprüft werden.
    echo ℹ️ Bitte stellen Sie sicher, dass Sie als Administrator ausführen.
    pause
    goto menu
)

:: Prüfe ob als Administrator ausgeführt
net session >nul 2>&1
if %errorlevel% neq 0 (
    echo ❌ Dieses Script muss als Administrator ausgeführt werden!
    pause
    goto menu
)

:: Prüfe ob XAMPP existiert
if not exist "%apacheBin%\httpd.exe" (
    echo ❌ XAMPP Apache wurde nicht gefunden!
    pause
    goto menu
)

:: Benutzereingaben
set /p projectName="🔧 Projektname eingeben (z.B. 'meinProjekt'): "
set /p domainName="🌐 Domainnamen eingeben (z.B. 'meinProjekt.local'): "
set /p projectTitle="📝 Projekttitel eingeben: "
set /p projectDesc="📋 Projektbeschreibung eingeben: "

:: Entferne Sonderzeichen aus Projektnamen
set "projectName=%projectName: =%"
set "projectName=%projectName:-=%"
set "projectName=%projectName:.=%"

:: Test Netzwerkzugriff auf hosts-Datei
echo 📝 Teste Schreibzugriff auf hosts-Datei...
copy /y "%hostsFile%" "%hostsFile%.test" >nul 2>&1
if %errorlevel% neq 0 (
    echo ⚠️ Keine Schreibrechte auf hosts-Datei!
    echo ℹ️ Bitte führen Sie das Script als Administrator aus.
    echo ℹ️ Prüfen Sie auch, ob ein Virenschutz den Zugriff blockiert.
    pause
    goto menu
)
del "%hostsFile%.test" >nul 2>&1

:: Backup von wichtigen Dateien
echo 📑 Erstelle Backups...
copy "%vhostsFile%" "%vhostsFile%.bak" >nul
copy "%sslConfFile%" "%sslConfFile%.bak" >nul
echo ✅ Backups erstellt

:: SSL-Verzeichnisse prüfen/erstellen
echo 📁 Prüfe SSL-Verzeichnisse...
if not exist "%apacheConf%\ssl.key" mkdir "%apacheConf%\ssl.key"
if not exist "%apacheConf%\ssl.crt" mkdir "%apacheConf%\ssl.crt"

:: Prüfe/Erstelle Standard-SSL-Zertifikate
echo 🔑 Prüfe Standard-SSL-Zertifikate...
if not exist "%apacheConf%\ssl.crt\server.crt" (
    echo Erstelle Standard-SSL-Zertifikate...
    "%apacheBin%\openssl.exe" req -new -newkey rsa:2048 -days 365 -nodes -x509 ^
        -subj "/C=DE/ST=State/L=City/O=XAMPP/CN=localhost" ^
        -keyout "%apacheConf%\ssl.key\server.key" ^
        -out "%apacheConf%\ssl.crt\server.crt" ^
        -config "%apacheConf%\openssl.cnf"
    
    if not exist "%apacheConf%\ssl.crt\server.crt" (
        echo ❌ Standard-SSL-Zertifikate konnten nicht erstellt werden!
        goto error
    )
    echo ✅ Standard-SSL-Zertifikate erstellt
) else (
    echo ✅ Standard-SSL-Zertifikate vorhanden
)

:: Projektverzeichnis erstellen
echo 📁 Erstelle Projektverzeichnis...
set projectPath=%htdocs%\%projectName%

if exist "%projectPath%" (
    echo ⚠️ Projektverzeichnis existiert bereits. Lösche alte Dateien...
    rd /s /q "%projectPath%"
)

mkdir "%projectPath%"
if not exist "%projectPath%" (
    echo ❌ Fehler beim Erstellen des Projektverzeichnisses!
    goto error
)

:: Erstelle project.json
echo 📝 Erstelle project.json...
(
    echo {
    echo     "title": "%projectTitle%",
    echo     "description": "%projectDesc%"
    echo }
) > "%projectPath%\project.json"

:: Erstelle index.php mit UTF-8 Header
echo 📝 Erstelle index.php...
(
    echo ^<?php
    echo header('Content-Type: text/html; charset=utf-8'^);
    echo ?^>
    echo ^<!DOCTYPE html^>
    echo ^<html lang="de"^>
    echo ^<head^>
    echo     ^<meta charset="UTF-8"^>
    echo     ^<title^>%projectTitle%^</title^>
    echo ^</head^>
    echo ^<body^>
    echo     ^<h1^>%projectTitle%^</h1^>
    echo     ^<p^>%projectDesc%^</p^>
    echo ^</body^>
    echo ^</html^>
) > "%projectPath%\index.php"

:: SSL-Zertifikate erstellen
echo 🔑 Erstelle Projekt-SSL-Zertifikate...
"%apacheBin%\openssl.exe" req -new -newkey rsa:2048 -days 365 -nodes -x509 ^
    -subj "/C=DE/ST=State/L=City/O=Organization/CN=%domainName%" ^
    -keyout "%apacheConf%\ssl.key\%domainName%.key" ^
    -out "%apacheConf%\ssl.crt\%domainName%.crt" ^
    -config "%apacheConf%\openssl.cnf"

:: Prüfe ob Zertifikate erstellt wurden
if not exist "%apacheConf%\ssl.key\%domainName%.key" (
    echo ❌ Projekt-SSL-Schlüssel konnte nicht erstellt werden!
    goto error
)
if not exist "%apacheConf%\ssl.crt\%domainName%.crt" (
    echo ❌ Projekt-SSL-Zertifikat konnte nicht erstellt werden!
    goto error
)
echo ✅ Projekt-SSL-Zertifikate erstellt

:: Host-Eintrag hinzufügen
echo 📝 Prüfe/Erstelle Host-Eintrag...
findstr /C:"%domainName%" "%hostsFile%" >nul
if errorlevel 1 (
   >> "%hostsFile%" echo 127.0.0.1 %domainName%
    if errorlevel 1 (
        echo ❌ Konnte Host-Eintrag nicht hinzufügen!
        goto error
    )
)
echo ✅ Host-Eintrag geprüft/erstellt

:: VirtualHost-Konfiguration erstellen
echo 📝 Erstelle VirtualHost-Einträge...
(
    echo.
    echo # VirtualHost für %domainName%
    echo ^<VirtualHost *:%httpPort%^>
    echo     ServerName %domainName%
    echo     DocumentRoot "%projectPath%"
    echo     ^<Directory "%projectPath%"^>
    echo         Options Indexes FollowSymLinks
    echo         AllowOverride All
    echo         Require all granted
    echo     ^</Directory^>
    echo     ErrorLog "logs/%domainName%-error.log"
    echo     CustomLog "logs/%domainName%-access.log" combined
    echo ^</VirtualHost^>
    echo.
    echo ^<VirtualHost *:%httpsPort%^>
    echo     ServerName %domainName%
    echo     DocumentRoot "%projectPath%"
    echo     SSLEngine on
    echo     SSLCertificateFile "%apacheConf%\ssl.crt\%domainName%.crt"
    echo     SSLCertificateKeyFile "%apacheConf%\ssl.key\%domainName%.key"
    echo     ^<Directory "%projectPath%"^>
    echo         Options Indexes FollowSymLinks
    echo         AllowOverride All
    echo         Require all granted
    echo     ^</Directory^>
    echo     ErrorLog "logs/%domainName%-ssl-error.log"
    echo     CustomLog "logs/%domainName%-ssl-access.log" combined
    echo ^</VirtualHost^>
) >> "%vhostsFile%"

:: Prüfe ob VirtualHost-Einträge geschrieben wurden
findstr /C:"ServerName %domainName%" "%vhostsFile%" >nul
if errorlevel 1 (
    echo ❌ VirtualHost-Einträge konnten nicht geschrieben werden!
    goto error
)
echo ✅ VirtualHost-Einträge erstellt

:: Prüfe SSL-Modul in httpd.conf
echo 📝 Prüfe SSL-Modul...
findstr /C:"LoadModule ssl_module modules/mod_ssl.so" "%apacheConf%\httpd.conf" >nul
if errorlevel 1 (
    echo LoadModule ssl_module modules/mod_ssl.so>> "%apacheConf%\httpd.conf"
    echo Include conf/extra/httpd-ssl.conf>> "%apacheConf%\httpd.conf"
    echo ✅ SSL-Modul aktiviert
)

:: Apache-Konfiguration testen
echo 🔍 Teste Apache-Konfiguration...
"%apacheBin%\httpd.exe" -t
if errorlevel 1 (
    echo ❌ Apache-Konfiguration fehlerhaft!
    goto error
)
echo ✅ Apache-Konfiguration ist valide

:: Erfolgsmeldung und URLs anzeigen
echo.
echo ✅ Projekt erfolgreich erstellt
echo.
echo 🔄 Bitte starten Sie Apache neu, um die Änderungen zu aktivieren
echo    Sie können dann Ihr Projekt unter folgenden URLs erreichen:
if "%httpPort%"=="80" (
    echo    http://%domainName%
) else (
    echo    http://%domainName%:%httpPort%
)
if "%httpsPort%"=="443" (
    echo    https://%domainName%
) else (
    echo    https://%domainName%:%httpsPort%
)
echo.
echo 📂 Projektverzeichnis: %projectPath%
echo.
pause
goto menu

:error
echo.
echo ❌ Es ist ein Fehler aufgetreten!
if exist "%vhostsFile%.bak" (
    copy "%vhostsFile%.bak" "%vhostsFile%" >nul 2>&1
    del "%vhostsFile%.bak" >nul 2>&1
)
if exist "%sslConfFile%.bak" (
    copy "%sslConfFile%.bak" "%sslConfFile%" >nul 2>&1
    del "%sslConfFile%.bak" >nul 2>&1
)
pause
goto menu